/*
	Avr.h
	
	Top class of the AVR micro controllers 
	Uros Platise (c) 1999
*/

#ifndef __AVR
#define __AVR

#include "Global.h"

#define AVR_MIN_VOLTAGE		2.7	/* V */
#define AVR_MAX_VOLTAGE		6.0	/* V */
#define AVR_DEFAULT_VOLTAGE	3.3	/* V */
#define AVR_VOLTAGE_SPRANGE	(5.0 - 3.2)	/* V */

class TAvr: public TDevice{
private:
  /* AVR Family Device (Part) List */
  struct TPart {
    const char* name;
    TByte part_family;
    TByte part_number;
    TAddr flash_size;
    TAddr eeprom_size;
    long t_wd_flash_32;		/* flash programming delay at 3.2 V */
    long t_wd_flash_50;    	/* flash programming delay at 5.0 V */
    long t_wd_eeprom_32;	/* ... */
    long t_wd_eeprom_50;
  };  
  
  static TPart parts[];
  TPart* part;
  bool device_locked;   
  long t_wd_flash;
  long t_wd_eeprom;

protected:
  enum TSegment{SEG_FLASH=0, SEG_EEPROM=1, SEG_FUSE=2};
  
  /* ATmega page programming */  
  bool page_mode_enabled;
  TAddr page_addr_fetched;	/* Becomes true when first byte is written */
  TAddr page_addr;		/* Fetched Page Address */

  
  /* Variables and Functions */
private:
  bool IsWritePageModeProvided();
  void SetWriteTimings();

protected:  
  TSegment segment;
  static const char* segment_names[];
  
  /* AVR Signs/Info */
  TByte vendor_code;
  TByte part_family;
  TByte part_number;
  
  void Identify();
  inline const char* GetPartName();  
  void CheckMemoryRange(TAddr addr);  
  long Get_t_wd_flash() const;
  long Get_t_wd_eeprom() const;    
  long Get_t_rst() const;

public:
  /* Set active segment. 
     Returns true if segment exists, otherwise false 
  */
  bool SetSegment(const char* segment_name);
  
    /* Returns char pointer of current active segment name.
  */
  const char* TellActiveSegment();
  
  /* Returns char pointer of the indexed segment name.
     Index is in range [0,no_of_segments].
     When index is out of range NULL is returned.
  */
  const char* ListSegment(unsigned index);

  /* Read byte description at address addr (as security bits) */
  const char* ReadByteDescription(TAddr addr);
  
  TAvr();
  ~TAvr();
};

#endif
